<?php

namespace Drupal\views_tokenized;

use Drupal\Core\Database\Query\AlterableInterface;
use Drupal\token\Token;

/**
 * Service for Token replacements in Views.
 */
class ViewsTokenized {

  /**
   * The Token service.
   *
   * @var \Drupal\token\Token
   */
  protected $token;

  /**
   * The ViewsTokenized constructor.
   *
   * @param \Drupal\token\Token $token
   *   The Token service.
   */
  public function __construct(Token $token) {
    $this->token = $token;
  }

  /**
   * Query alteration that replaces any Tokens.
   *
   * @param \Drupal\Core\Database\Query\AlterableInterface $query
   *   A Query object describing the composite parts of a SQL query.
   */
  public function alterQuery(AlterableInterface $query) {
    $to_replace = [];
    $this->collectTokenInput($query, $to_replace);
    foreach ($to_replace as &$text) {
      $text = trim($this->token->replace($text));
    }
  }

  /**
   * Collects input containing Tokens from the given query.
   *
   * @param \Drupal\Core\Database\Query\AlterableInterface $query
   *   A Query object describing the composite parts of a SQL query.
   * @param array &$to_replace
   *   A list of already collected input to evaluate.
   */
  protected function collectTokenInput(AlterableInterface $query, array &$to_replace) {
    /** @var \Drupal\Core\Database\Query\Select $query */
    $tables = &$query->getTables();
    $conditions = &$query->conditions();

    foreach ($tables as $table_name => $table_metadata) {
      if (empty($table_metadata['arguments'])) {
        continue;
      }
      foreach ($table_metadata['arguments'] as $replacement_key => $value) {
        if (is_array($value)) {
          foreach ($value as $sub_key => $sub_value) {
            if ($this->containsToken($sub_value)) {
              $to_replace[] = &$tables[$table_name]['arguments'][$replacement_key][$sub_key];
            }
          }
        }
        elseif ($this->containsToken($value)) {
          $to_replace[] = &$tables[$table_name]['arguments'][$replacement_key];
        }
      }
    }

    $this->collectFromConditions($query, $conditions, $to_replace);
  }

  /**
   * Sub-method to collect input contaning Tokens from conditions.
   *
   * @param \Drupal\Core\Database\Query\AlterableInterface $query
   *   A Query object describing the composite parts of a SQL query.
   * @param mixed &$conditions
   *   The conditions.
   * @param array &$to_replace
   *   A list of already collected input to evaluate.
   */
  protected function collectFromConditions(AlterableInterface $query, &$conditions, array &$to_replace) {
    foreach ($conditions as $condition_id => &$condition) {
      if (!is_numeric($condition_id)) {
        continue;
      }
      if (is_string($condition['field']) && $this->containsToken($condition['field'])) {
        $to_replace[] = &$condition['field'];
      }
      elseif (is_object($condition['field'])) {
        $sub_conditions = &$condition['field']->conditions();
        $this->collectFromConditions($query, $sub_conditions, $to_replace);
      }
      if (is_object($condition['value'])) {
        $this->collectTokenInput($condition['value'], $to_replace);
      }
      elseif (isset($condition['value'])) {
        if (is_array($condition['value'])) {
          foreach ($condition['value'] as $key => $value) {
            if (is_string($value) && $this->containsToken($value)) {
              $to_replace[] = &$condition['value'][$key];
            }
          }
        }
        elseif (is_string($condition['value']) && $this->containsToken($condition['value'])) {
          $to_replace[] = &$condition['value'];
        }
      }
    }
  }

  /**
   * Checks whether the given string contains at least one Token.
   *
   * @param string $string
   *   The string to check for.
   *
   * @return bool
   *   Returns TRUE if it contains at least one Token, FALSE otherwise.
   */
  public function containsToken($string) {
    if (is_object($string) && method_exists($string, '__toString')) {
      $string = (string) $string;
    }
    return is_string($string)
      && strpos($string, '[') !== FALSE
      && strpos($string, ':')
      && strpos($string, ']')
      && $this->token->scan($string);
  }

}
